@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -19,8 +19,8 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import os, sys, ConfigParser, gtk, pango, subprocess, base64
-import cons
+import os, sys, ConfigParser, gtk, pango, locale, subprocess, base64
+import cons, pgsc_spellcheck
 
 ICONS_SIZE = {1: gtk.ICON_SIZE_MENU, 2: gtk.ICON_SIZE_SMALL_TOOLBAR, 3: gtk.ICON_SIZE_LARGE_TOOLBAR,
               4: gtk.ICON_SIZE_DND, 5: gtk.ICON_SIZE_DIALOG}
@@ -28,7 +28,13 @@ ICONS_SIZE = {1: gtk.ICON_SIZE_MENU, 2: gtk.ICON_SIZE_SMALL_TOOLBAR, 3: gtk.ICON
 LINK_CUSTOM_ACTION_DEFAULT_WEB = "firefox %s &"
 LINK_CUSTOM_ACTION_DEFAULT_FILE = "xdg-open %s &"
 HORIZONTAL_RULE = "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+COLOR_PALETTE_DEFAULT = ["#000000", "#ffffff", "#7f7f7f", "#ff0000", "#a020f0",
+                         "#0000ff", "#add8e6", "#00ff00", "#ffff00", "#ffa500",
+                         "#e6e6fa", "#a52a2a", "#8b6914", "#1e90ff", "#ffc0cb",
+                         "#90ee90", "#1a1a1a", "#4d4d4d", "#bfbfbf", "#e5e5e5"]
+SPECIAL_CHARS_DEFAULT = "“”„•☐☑☒…‰€©®™°↓↑→←↔↵⇓⇑⇒⇐⇔»«▼▲►◄≤≥≠±¹²³½¼⅛×÷∞ø∑√∫ΔδΠπΣΦΩωαβγεηλμ☺☻☼♥♀♂♪♫"
 
+SPELL_CHECK_LANG_DEFAULT = locale.getdefaultlocale()[0]
 
 def config_file_load(inst):
     """Load the Preferences from Config File"""
@@ -37,14 +43,15 @@ def config_file_load(inst):
         config.read(cons.CONFIG_PATH)
         
         section = "state"
-        inst.file_dir = config.get(section, "file_dir") if config.has_option(section, "file_dir") else ""
-        inst.file_name = config.get(section, "file_name") if config.has_option(section, "file_name") else ""
+        inst.file_dir = unicode(config.get(section, "file_dir"), cons.STR_UTF8, cons.STR_IGNORE) if config.has_option(section, "file_dir") else ""
+        inst.file_name = unicode(config.get(section, "file_name"), cons.STR_UTF8, cons.STR_IGNORE) if config.has_option(section, "file_name") else ""
         inst.toolbar_visible = config.getboolean(section, "toolbar_visible") if config.has_option(section, "toolbar_visible") else True
         inst.win_is_maximized = config.getboolean(section, "win_is_maximized") if config.has_option(section, "win_is_maximized") else False
         # restore window size and position
         if config.has_option(section, "win_position_x") and config.has_option(section, "win_position_y"):
-            win_position = [config.getint(section, "win_position_x"), config.getint(section, "win_position_y")]
-            inst.window.move(win_position[0], win_position[1])
+            inst.win_position = [config.getint(section, "win_position_x"), config.getint(section, "win_position_y")]
+            inst.window.move(inst.win_position[0], inst.win_position[1])
+        else: inst.win_position = [10, 10]
         if inst.win_is_maximized: inst.window.maximize()
         elif config.has_option(section, "win_size_w") and config.has_option(section, "win_size_h"):
             win_size = [config.getint(section, "win_size_w"), config.getint(section, "win_size_h")]
@@ -62,14 +69,16 @@ def config_file_load(inst):
         if config.has_option(section, "recent_docs"):
             temp_recent_docs = config.get(section, "recent_docs").split(cons.CHAR_SPACE)
             for element in temp_recent_docs:
-                if element: inst.recent_docs.append(base64.b64decode(element))
+                if element: inst.recent_docs.append(unicode(base64.b64decode(element), cons.STR_UTF8, cons.STR_IGNORE))
         inst.pick_dir = config.get(section, "pick_dir") if config.has_option(section, "pick_dir") else ""
-        inst.link_type = config.get(section, "link_type") if config.has_option(section, "link_type") else "webs"
+        inst.link_type = config.get(section, "link_type") if config.has_option(section, "link_type") else cons.LINK_TYPE_WEBS
         inst.show_node_name_label = config.getboolean(section, "show_node_name_label") if config.has_option(section, "show_node_name_label") else True
         if config.has_option(section, "toolbar_icon_size"):
             inst.toolbar_icon_size = config.getint(section, "toolbar_icon_size")
             if inst.toolbar_icon_size not in ICONS_SIZE: inst.toolbar_icon_size = 1
         else: inst.toolbar_icon_size = 1
+        inst.curr_colors = {'f':gtk.gdk.color_parse(config.get(section, "fg")) if config.has_option(section, "fg") else None,
+                            'b':gtk.gdk.color_parse(config.get(section, "bg")) if config.has_option(section, "bg") else None}
         
         section = "tree"
         inst.rest_exp_coll = config.getint(section, "rest_exp_coll") if config.has_option(section, "rest_exp_coll") else 0
@@ -81,6 +90,8 @@ def config_file_load(inst):
         section = "editor"
         inst.syntax_highlighting = config.get(section, "syntax_highlighting") if config.has_option(section, "syntax_highlighting") else cons.CUSTOM_COLORS_ID
         inst.style_scheme = config.get(section, "style_scheme") if config.has_option(section, "style_scheme") else cons.STYLE_SCHEME_DEFAULT
+        inst.enable_spell_check = config.getboolean(section, "enable_spell_check") if config.has_option(section, "enable_spell_check") else False
+        inst.spell_check_lang = config.get(section, "spell_check_lang") if config.has_option(section, "spell_check_lang") else SPELL_CHECK_LANG_DEFAULT
         inst.show_line_numbers = config.getboolean(section, "show_line_numbers") if config.has_option(section, "show_line_numbers") else False
         inst.spaces_instead_tabs = config.getboolean(section, "spaces_instead_tabs") if config.has_option(section, "spaces_instead_tabs") else True
         inst.tabs_width = config.getint(section, "tabs_width") if config.has_option(section, "tabs_width") else 4
@@ -90,6 +101,7 @@ def config_file_load(inst):
         inst.show_white_spaces = config.getboolean(section, "show_white_spaces") if config.has_option(section, "show_white_spaces") else True
         inst.highl_curr_line = config.getboolean(section, "highl_curr_line") if config.has_option(section, "highl_curr_line") else True
         inst.h_rule = config.get(section, "h_rule") if config.has_option(section, "h_rule") else HORIZONTAL_RULE
+        inst.special_chars = unicode(config.get(section, "special_chars") if config.has_option(section, "special_chars") else SPECIAL_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
         inst.timestamp_format = config.get(section, "timestamp_format") if config.has_option(section, "timestamp_format") else "%Y/%m/%d - %H:%M"
         if config.has_option(section, "weblink_custom_action"):
             temp_str = config.get(section, "weblink_custom_action")
@@ -132,6 +144,9 @@ def config_file_load(inst):
         inst.rt_def_bg = config.get(section, "rt_def_bg") if config.has_option(section, "rt_def_bg") else cons.RICH_TEXT_DARK_BG
         inst.tt_def_fg = config.get(section, "tt_def_fg") if config.has_option(section, "tt_def_fg") else cons.TREE_TEXT_LIGHT_FG
         inst.tt_def_bg = config.get(section, "tt_def_bg") if config.has_option(section, "tt_def_bg") else cons.TREE_TEXT_LIGHT_BG
+        if config.has_option(section, "palette_list"):
+            inst.palette_list = config.get(section, "palette_list").split(":")
+        else: inst.palette_list = COLOR_PALETTE_DEFAULT
         
         section = "misc"
         inst.systray = config.getboolean(section, "systray") if config.has_option(section, "systray") else False
@@ -141,7 +156,8 @@ def config_file_load(inst):
             inst.autosave = [config.getboolean(section, "autosave"), config.getint(section, "autosave_val")]
         else: inst.autosave = [False, 5]
         inst.check_version = config.getboolean(section, "check_version") if config.has_option(section, "check_version") else False
-        inst.enable_mod_time_sentinel = config.getboolean(section, "mod_time_sent") if config.has_option(section, "mod_time_sent") else False
+        inst.reload_doc_last = config.getboolean(section, "reload_doc_last") if config.has_option(section, "reload_doc_last") else True
+        inst.enable_mod_time_sentinel = config.getboolean(section, "mod_time_sent") if config.has_option(section, "mod_time_sent") else True
         inst.backup_copy = config.getboolean(section, "backup_copy") if config.has_option(section, "backup_copy") else True
         inst.autosave_on_quit = config.getboolean(section, "autosave_on_quit") if config.has_option(section, "autosave_on_quit") else False
         inst.limit_undoable_steps = config.getint(section, "limit_undoable_steps") if config.has_option(section, "limit_undoable_steps") else 20
@@ -149,6 +165,7 @@ def config_file_load(inst):
         inst.file_dir = ""
         inst.file_name = ""
         inst.node_path = None
+        inst.curr_colors = {'f':None, 'b':None}
         inst.syntax_highlighting = cons.CUSTOM_COLORS_ID
         inst.style_scheme = cons.STYLE_SCHEME_DEFAULT
         inst.tree_font = "Sans 8" # default tree font
@@ -158,7 +175,11 @@ def config_file_load(inst):
         inst.rt_def_bg = cons.RICH_TEXT_DARK_BG
         inst.tt_def_fg = cons.TREE_TEXT_LIGHT_FG
         inst.tt_def_bg = cons.TREE_TEXT_LIGHT_BG
+        inst.palette_list = COLOR_PALETTE_DEFAULT
         inst.h_rule = HORIZONTAL_RULE
+        inst.special_chars = unicode(SPECIAL_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
+        inst.enable_spell_check = False
+        inst.spell_check_lang = SPELL_CHECK_LANG_DEFAULT
         inst.show_line_numbers = False
         inst.spaces_instead_tabs = True
         inst.tabs_width = 4
@@ -166,12 +187,13 @@ def config_file_load(inst):
         inst.line_wrapping = True
         inst.auto_indent = True
         inst.systray = False
+        inst.win_position = [10, 10]
         inst.autosave = [False, 5]
         inst.win_is_maximized = False
         inst.rest_exp_coll = 0
         inst.expanded_collapsed_string = ""
         inst.pick_dir = ""
-        inst.link_type = "webs"
+        inst.link_type = cons.LINK_TYPE_WEBS
         inst.toolbar_icon_size = 1
         inst.table_rows = 3
         inst.table_columns = 3
@@ -189,7 +211,8 @@ def config_file_load(inst):
         inst.glade.spinbutton_codebox_width.set_value(700)
         inst.glade.spinbutton_codebox_height.set_value(100)
         inst.check_version = False
-        inst.enable_mod_time_sentinel = False
+        inst.reload_doc_last = True
+        inst.enable_mod_time_sentinel = True
         inst.backup_copy = True
         inst.autosave_on_quit = False
         inst.tree_right_side = False
@@ -209,7 +232,7 @@ def config_file_apply(inst):
     else: user_active_restore = False
     # treeview
     inst.hpaned.set_property('position', inst.hpaned_pos)
-    inst.header_node_name_label.set_visible(inst.show_node_name_label)
+    inst.header_node_name_label.set_property(cons.STR_VISIBLE, inst.show_node_name_label)
     inst.set_treeview_font()
     inst.glade.fontbutton_tree.set_font_name(inst.tree_font)
     # sourceview
@@ -240,6 +263,16 @@ def config_file_apply(inst):
     inst.treeview.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(inst.tt_def_bg))
     inst.treeview.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(inst.tt_def_fg))
     inst.glade.entry_horizontal_rule.set_text(inst.h_rule)
+    inst.glade.textbuffer_special_chars.set_text(inst.special_chars)
+    inst.glade.textbuffer_special_chars.set_modified(False)
+    if not pgsc_spellcheck.HAS_PYENCHANT:
+        inst.enable_spell_check = False
+        inst.glade.checkbutton_spell_check.set_sensitive(False)
+    inst.glade.checkbutton_spell_check.set_active(inst.enable_spell_check)
+    if inst.enable_spell_check:
+        inst.spell_check_set_on()
+    else:
+        inst.glade.combobox_spell_check_lang.set_sensitive(False)
     inst.sourceview.set_show_line_numbers(inst.show_line_numbers)
     inst.glade.checkbutton_line_nums.set_active(inst.show_line_numbers)
     inst.sourceview.set_insert_spaces_instead_of_tabs(inst.spaces_instead_tabs)
@@ -261,6 +294,7 @@ def config_file_apply(inst):
     inst.glade.radiobutton_nodes_startup_expand.set_active(inst.rest_exp_coll == 1)
     inst.glade.radiobutton_nodes_startup_collapse.set_active(inst.rest_exp_coll == 2)
     inst.glade.checkbutton_newer_version.set_active(inst.check_version)
+    inst.glade.checkbutton_reload_doc_last.set_active(inst.reload_doc_last)
     inst.glade.checkbutton_mod_time_sentinel.set_active(inst.enable_mod_time_sentinel)
     inst.glade.checkbutton_backup_before_saving.set_active(inst.backup_copy)
     inst.glade.checkbutton_autosave_on_quit.set_active(inst.autosave_on_quit)
@@ -281,9 +315,6 @@ def config_file_apply(inst):
     inst.glade.entry_custom_folderlink_cmd.set_text(inst.folderlink_custom_action[1])
     inst.glade.entry_timestamp_format.set_text(inst.timestamp_format)
     #
-    inst.glade.radiobutton_link_website.set_active(inst.link_type == "webs")
-    inst.glade.radiobutton_link_node_anchor.set_active(inst.link_type == "node")
-    inst.glade.radiobutton_link_file.set_active(inst.link_type == "file")
     inst.glade.table_column_rename_radiobutton.set_active(inst.table_column_mode == "rename")
     inst.glade.table_column_delete_radiobutton.set_active(inst.table_column_mode == "delete")
     inst.glade.table_column_add_radiobutton.set_active(inst.table_column_mode == "add")
@@ -299,7 +330,7 @@ def config_file_apply(inst):
     inst.glade.spinbutton_limit_undoable_steps.set_value(inst.limit_undoable_steps)
     inst.glade.spinbutton_tree_nodes_names_width.set_value(inst.cherry_wrap_width)
     inst.renderer_text.set_property('wrap-width', inst.cherry_wrap_width)
-    inst.ui.get_widget("/ToolBar").set_visible(inst.toolbar_visible)
+    inst.ui.get_widget("/ToolBar").set_property(cons.STR_VISIBLE, inst.toolbar_visible)
     inst.ui.get_widget("/ToolBar").set_style(gtk.TOOLBAR_ICONS)
     inst.ui.get_widget("/ToolBar").set_property("icon-size", ICONS_SIZE[inst.toolbar_icon_size])
     if inst.autosave[0]: inst.autosave_timer_start()
@@ -313,11 +344,11 @@ def config_file_save(inst):
     config.add_section(section)
     config.set(section, "file_dir", inst.file_dir)
     config.set(section, "file_name", inst.file_name)
-    config.set(section, "toolbar_visible", inst.ui.get_widget("/ToolBar").get_visible())
+    config.set(section, "toolbar_visible", inst.ui.get_widget("/ToolBar").get_property(cons.STR_VISIBLE))
     config.set(section, "win_is_maximized", inst.win_is_maximized)
-    win_position = inst.window.get_position()
-    config.set(section, "win_position_x", win_position[0])
-    config.set(section, "win_position_y", win_position[1])
+    inst.win_position = inst.window.get_position()
+    config.set(section, "win_position_x", inst.win_position[0])
+    config.set(section, "win_position_y", inst.win_position[1])
     if not inst.win_is_maximized:
         win_size = inst.window.get_size()
         config.set(section, "win_size_w", win_size[0])
@@ -331,15 +362,18 @@ def config_file_save(inst):
         config.set(section, "cursor_position", inst.curr_buffer.get_property(cons.STR_CURSOR_POSITION))
     if inst.recent_docs:
         temp_recent_docs = []
-        for element in inst.recent_docs:
+        for i, element in enumerate(inst.recent_docs):
+            if i >= cons.MAX_RECENT_DOCS: break
             temp_recent_docs.append(base64.b64encode(element))
         str_recent_docs = cons.CHAR_SPACE.join(temp_recent_docs)
     else: str_recent_docs = ""
     config.set(section, "recent_docs", str_recent_docs)
     config.set(section, "pick_dir", inst.pick_dir)
     config.set(section, "link_type", inst.link_type)
-    config.set(section, "show_node_name_label", inst.header_node_name_label.get_visible())
+    config.set(section, "show_node_name_label", inst.header_node_name_label.get_property(cons.STR_VISIBLE))
     config.set(section, "toolbar_icon_size", inst.toolbar_icon_size)
+    if inst.curr_colors['f']: config.set(section, "fg", inst.curr_colors['f'].to_string())
+    if inst.curr_colors['b']: config.set(section, "bg", inst.curr_colors['b'].to_string())
     
     section = "tree"
     config.add_section(section)
@@ -355,6 +389,8 @@ def config_file_save(inst):
     config.add_section(section)
     config.set(section, "syntax_highlighting", inst.syntax_highlighting)
     config.set(section, "style_scheme", inst.style_scheme)
+    config.set(section, "spell_check_lang", inst.spell_check_lang)
+    config.set(section, "enable_spell_check", inst.enable_spell_check)
     config.set(section, "show_line_numbers", inst.show_line_numbers)
     config.set(section, "spaces_instead_tabs", inst.spaces_instead_tabs)
     config.set(section, "tabs_width", inst.tabs_width)
@@ -364,6 +400,7 @@ def config_file_save(inst):
     config.set(section, "show_white_spaces", inst.show_white_spaces)
     config.set(section, "highl_curr_line", inst.highl_curr_line)
     config.set(section, "h_rule", inst.h_rule)
+    config.set(section, "special_chars", inst.special_chars)
     config.set(section, "timestamp_format", inst.timestamp_format)
     config.set(section, "weblink_custom_action", str(inst.weblink_custom_action[0])+inst.weblink_custom_action[1])
     config.set(section, "filelink_custom_action", str(inst.filelink_custom_action[0])+inst.filelink_custom_action[1])
@@ -395,6 +432,7 @@ def config_file_save(inst):
     config.set(section, "rt_def_bg", inst.rt_def_bg)
     config.set(section, "tt_def_fg", inst.tt_def_fg)
     config.set(section, "tt_def_bg", inst.tt_def_bg)
+    config.set(section, "palette_list", ":".join(inst.palette_list))
     
     section = "misc"
     config.add_section(section)
@@ -404,6 +442,7 @@ def config_file_save(inst):
     config.set(section, "autosave", inst.autosave[0])
     config.set(section, "autosave_val", inst.autosave[1])
     config.set(section, "check_version", inst.check_version)
+    config.set(section, "reload_doc_last", inst.reload_doc_last)
     config.set(section, "mod_time_sent", inst.enable_mod_time_sentinel)
     config.set(section, "backup_copy", inst.backup_copy)
     config.set(section, "autosave_on_quit", inst.autosave_on_quit)
