@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -22,7 +22,7 @@
 import os, sys
 
 
-VERSION = "0.29"
+VERSION = "0.30.5"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
 if sys.platform[0:3] == "win":
@@ -49,7 +49,7 @@ CONFIG_PATH = os.path.join(CONFIG_DIR, 'config.cfg')
 LANG_PATH = os.path.join(CONFIG_DIR, 'lang')
 IMG_PATH = os.path.join(CONFIG_DIR, 'img_tmp.png')
 
-AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es', 'fr', 'it', 'pl', 'ru', 'uk', 'zh_CN']
+AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es', 'fr', 'it', 'pl', 'pt_BR', 'ru', 'uk', 'zh_CN']
 COLOR_48_YELLOW = "#bbbbbbbb0000"
 COLOR_48_WHITE = "#ffffffffffff"
 COLOR_48_BLACK = "#000000000000"
@@ -67,7 +67,7 @@ TREE_TEXT_DARK_BG = COLOR_24_BLUEBG
 TREE_TEXT_LIGHT_FG = COLOR_24_LBLACK
 TREE_TEXT_LIGHT_BG = COLOR_24_GRAY
 BAD_CHARS = "[\x00-\x08\x0b-\x1f]"
-
+GTKSPELLCHECK_TAG_NAME = "gtkspellchecker-misspelled"
 TAG_WEIGHT = "weight"
 TAG_FOREGROUND = "foreground"
 TAG_BACKGROUND = "background"
@@ -93,7 +93,10 @@ TAG_PROP_CENTER = "center"
 TAG_PROP_RIGHT = "right"
 TAG_PROPERTIES = [TAG_WEIGHT, TAG_FOREGROUND, TAG_BACKGROUND, TAG_STYLE, TAG_UNDERLINE,
                   TAG_STRIKETHROUGH, TAG_SCALE, TAG_FAMILY, TAG_JUSTIFICATION, TAG_LINK]
-
+LINK_TYPE_WEBS = "webs"
+LINK_TYPE_FILE = "file"
+LINK_TYPE_FOLD = "fold"
+LINK_TYPE_NODE = "node"
 CUSTOM_COLORS_ID = "custom-colors"
 STYLE_SCHEME_DEFAULT = "cobalt"
 ANCHOR_CHAR = GLADE_PATH + 'anchor.png'
@@ -102,19 +105,25 @@ NODES_ICONS = {0:'cherry_red', 1:'cherry_blue', 2:'cherry_orange', 3:'cherry_cya
 CODE_ICONS = {"python": 'cherry_green', "sh":'cherry_purple',
               "c":'cherry_black', "cpp":'cherry_black', "chdr":'cherry_black'}
 
+MAX_FILE_NAME_LEN = 150
 WHITE_SPACE_BETW_PIXB_AND_TEXT = 3
 GRID_SLIP_OFFSET = 3
 MAIN_WIN_TO_TEXT_WIN_NORMALIZER = 50
+MAX_RECENT_DOCS = 10
+TABLE_DEFAULT_COL_MIN = 40
+TABLE_DEFAULT_COL_MAX = 400
 
 CHAR_SPACE = " "
 CHAR_NEWLINE = "\n"
 CHAR_CR = "\r"
 CHAR_TAB = "\t"
 CHAR_LISTBUL = "•"
+CHAR_LISTARR = "►"
 CHAR_TILDE = "~"
 CHAR_MINUS = "-"
 CHAR_DQUOTE = '"'
 CHAR_SQUOTE = "'"
+CHAR_SLASH = "/"
 CHAR_BSLASH = "\\"
 CHAR_SQ_BR_OPEN = "["
 CHAR_X = "X"
@@ -124,7 +133,8 @@ CHAR_PARENTH_CLOSE = ")"
 CHAR_LESSER = "<"
 CHAR_GREATER = ">"
 CHAR_STAR = "*"
-CHAR_SLASH = "/"
+CHAR_QUESTION = "?"
+CHAR_COLON = ":"
 CHAR_USCORE = "_"
 CHAR_EQUAL = "="
 CHAR_BR_OPEN = "{"
@@ -141,6 +151,10 @@ SPECIAL_CHAR_REGISTERED_TRADEMARK = "®"
 
 STR_CURSOR_POSITION = "cursor-position"
 STR_STOCK_CT_IMP = "import_in_cherrytree"
+STR_VISIBLE = "visible"
+STR_UTF8 = "utf-8"
+STR_UTF16 = "utf-16"
+STR_IGNORE = "ignore"
 
 HTML_HEADER = '<!doctype html><html><head><meta http-equiv="content-type" content="text/html; charset=utf-8"><title>%s</title><meta name="generator" content="CherryTree"></head><body>'
 HTML_FOOTER = '</body></html>'
@@ -320,6 +334,8 @@ UI_INFO = """
                 <menuitem action='ToggleCase'/>
             </menu>
             <separator/>
+            <menuitem action='PastePlainText'/>
+            <separator/>
             <menuitem action='CutRow'/>
             <menuitem action='CopyRow'/>
             <menuitem action='DeleteRow'/>
@@ -380,6 +396,9 @@ UI_INFO = """
             <menuitem action='ShowAllMatchesDialog'/>
             <separator/>
             <menuitem action='ToggleTreeText'/>
+            <menuitem action='ToggleNodeExpColl'/>
+            <menuitem action='NodesExpAll'/>
+            <menuitem action='NodesCollAll'/>
             <separator/>
             <menuitem action='IncreaseToolbarIconsSize'/>
             <menuitem action='DecreaseToolbarIconsSize'/>
@@ -397,6 +416,7 @@ UI_INFO = """
             <menuitem action='FromCherryTree'/>
             <menuitem action='FromGnote'/>
             <menuitem action='FromKeepNote'/>
+            <menuitem action='FromKeyNote'/>
             <menuitem action='FromKnowit'/>
             <menuitem action='FromLeo'/>
             <menuitem action='FromMempad'/>
@@ -409,7 +429,8 @@ UI_INFO = """
 
         <menu action='ExportMenu'>
             <menuitem action='NodePrint'/>
-            <menuitem action='Export2Txt'/>
+            <menuitem action='Export2TxtMultiple'/>
+            <menuitem action='Export2TxtSingle'/>
             <menuitem action='Export2HTML'/>
             <menuitem action='Export2CTD'/>
         </menu>
@@ -532,7 +553,8 @@ def get_entries(inst):
     ( "Save", "gtk-save", _("_Save"), "<control>S", _("Save File"), inst.file_save),
     ( "SaveAs", "gtk-save-as", _("Save _As"), "<control><shift>S", _("Save File As"), inst.file_save_as),
     ( "Export2HTML", "to_html", _("Export To _HTML"), None, _("Export To HTML"), inst.export_to_html),
-    ( "Export2Txt", "to_txt", _("Export to Plain _Text"), None, _("Export to Plain Text"), inst.export_to_txt),
+    ( "Export2TxtMultiple", "to_txt", _("Export to Multiple Plain _Text Files"), None, _("Export to Multiple Plain Text Files"), inst.export_to_txt_multiple),
+    ( "Export2TxtSingle", "to_txt", _("Export to _Single Plain Text File"), None, _("Export to Single Plain Text File"), inst.export_to_txt_single),
     ( "Export2CTD", "export_from_cherrytree", _("_Export To CherryTree Document"), None, _("Export To CherryTree Document"), inst.export_to_ctd),
     ( "PageSetup", "gtk-print", _("Pa_ge Setup"), "<control><shift>P", _("Set up the Page for Printing"), inst.export_print_page_setup),
     ( "NodePrint", "gtk-print", _("_Print / Export To PDF"), "<control>P", _("Print / Export To PDF"), inst.export_print),
@@ -574,7 +596,8 @@ def get_entries(inst):
     ( "Redo", "gtk-redo", _("_Redo"), "<control>Y", _("Redo Previously Discarded Operation"), inst.requested_step_ahead),
     ( "InheritSyntax", "gtk-execute", _("_Inherit Syntax"), None, _("Change the Selected Node's Children Syntax Highlighting to the Father's Syntax Highlighting"), inst.node_inherit_syntax),
     ( "FormatLatest", "format_text_latest", _("Format _Latest"), "F7", _("Memory of Latest Text Format Type"), inst.apply_tag_latest),
-    ( "RemoveFormatting", "gtk-clear", _("_Remove Formatting"), None, _("Remove the Formatting from the Selected Text"), inst.remove_text_formatting),
+    ( "PastePlainText", "edit-paste", _("_Paste as Plain Text"), "<control><shift>V", _("Paste as Plain Text, Discard the Rich Text Formatting"), inst.paste_as_plain_text),
+    ( "RemoveFormatting", "gtk-clear", _("_Remove Formatting"), "<control><shift>R", _("Remove the Formatting from the Selected Text"), inst.remove_text_formatting),
     ( "ColorForeground", "color_foreground", _("Text _Color Foreground"), None, _("Change the Color of the Selected Text Foreground"), inst.apply_tag_foreground),
     ( "ColorBackground", "color_background", _("Text C_olor Background"), None, _("Change the Color of the Selected Text Background"), inst.apply_tag_background),
     ( "Bold", "format-text-bold", _("Toggle _Bold Property"), "<control>B", _("Toggle Bold Property of the Selected Text"), inst.apply_tag_bold),
@@ -607,20 +630,24 @@ def get_entries(inst):
     ( "ShowAllMatchesDialog", "find", _("Show _All Matches Dialog"), "<control><shift>A", _("Show Search All Matches Dialog"), inst.find_allmatchesdialog_restore),
     ( "IncreaseToolbarIconsSize", "gtk-add", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
     ( "DecreaseToolbarIconsSize", "gtk-remove", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
-    ( "ToggleTreeText", "gtk-jump-to", _("Toggle Focus Tree/Text"), "<control>J", _("Toggle Focus Between Tree and Text"), inst.toggle_tree_text),
+    ( "ToggleTreeText", "gtk-jump-to", _("Toggle _Focus Tree/Text"), "<control>J", _("Toggle Focus Between Tree and Text"), inst.toggle_tree_text),
+    ( "ToggleNodeExpColl", "gtk-zoom-in", _("Toggle Node _Expanded/Collapsed"), "<control><shift>J", _("Toggle Expanded/Collapsed Status of the Selected Node"), inst.toggle_tree_node_expanded_collapsed),
+    ( "NodesExpAll", "gtk-zoom-in", _("E_xpand All Nodes"), None, _("Expand All the Tree Nodes"), inst.nodes_expand_all),
+    ( "NodesCollAll", "gtk-zoom-out", _("_Collapse All Nodes"), None, _("Collapse All the Tree Nodes"), inst.nodes_collapse_all),
     ( "Fullscreen", "gtk-fullscreen", _("_Full Screen On/Off"), "F11", _("Toggle Full Screen On/Off"), inst.fullscreen_toggle),
     ( "FromBasket", STR_STOCK_CT_IMP, _("From _Basket Folder"), None, _("Add Nodes of a Basket Folder to the Current Tree"), inst.nodes_add_from_basket_folder),
     ( "FromCherryTree", STR_STOCK_CT_IMP, _("From _CherryTree File"), None, _("Add Nodes of a CherryTree File to the Current Tree"), inst.nodes_add_from_cherrytree_file),
-    ( "FromNoteCase", STR_STOCK_CT_IMP, _("From _NoteCase File"), None, _("Add Nodes of a NoteCase File to the Current Tree"), inst.nodes_add_from_notecase_file),
-    ( "FromKeepNote", STR_STOCK_CT_IMP, _("From _KeepNote Folder"), None, _("Add Nodes of a KeepNote Folder to the Current Tree"), inst.nodes_add_from_keepnote_folder),
-    ( "FromKnowit", STR_STOCK_CT_IMP, _("From K_nowit File"), None, _("Add Nodes of a Knowit File to the Current Tree"), inst.nodes_add_from_knowit_file),
-    ( "FromZim", STR_STOCK_CT_IMP, _("From _Zim Folder"), None, _("Add Nodes of a Zim Folder to the Current Tree"), inst.nodes_add_from_zim_folder),
-    ( "FromTomboy", STR_STOCK_CT_IMP, _("From T_omboy Folder"), None, _("Add Nodes of a Tomboy Folder to the Current Tree"), inst.nodes_add_from_tomboy_folder),
     ( "FromGnote", STR_STOCK_CT_IMP, _("From _Gnote Folder"), None, _("Add Nodes of a Gnote Folder to the Current Tree"), inst.nodes_add_from_gnote_folder),
-    ( "FromTuxCards", STR_STOCK_CT_IMP, _("From _TuxCards File"), None, _("Add Nodes of a TuxCards File to the Current Tree"), inst.nodes_add_from_tuxcards_file),
-    ( "FromTreepad", STR_STOCK_CT_IMP, _("From T_reepad Lite File"), None, _("Add Nodes of a Treepad Lite File to the Current Tree"), inst.nodes_add_from_treepad_file),
-    ( "FromMempad", STR_STOCK_CT_IMP, _("From _Mempad File"), None, _("Add Nodes of a Mempad File to the Current Tree"), inst.nodes_add_from_mempad_file),
+    ( "FromKeepNote", STR_STOCK_CT_IMP, _("From _KeepNote Folder"), None, _("Add Nodes of a KeepNote Folder to the Current Tree"), inst.nodes_add_from_keepnote_folder),
+    ( "FromKeyNote", STR_STOCK_CT_IMP, _("From K_eyNote File"), None, _("Add Nodes of a KeyNote File to the Current Tree"), inst.nodes_add_from_keynote_file),
+    ( "FromKnowit", STR_STOCK_CT_IMP, _("From K_nowit File"), None, _("Add Nodes of a Knowit File to the Current Tree"), inst.nodes_add_from_knowit_file),
     ( "FromLeo", STR_STOCK_CT_IMP, _("From _Leo File"), None, _("Add Nodes of a Leo File to the Current Tree"), inst.nodes_add_from_leo_file),
+    ( "FromMempad", STR_STOCK_CT_IMP, _("From _Mempad File"), None, _("Add Nodes of a Mempad File to the Current Tree"), inst.nodes_add_from_mempad_file),
+    ( "FromNoteCase", STR_STOCK_CT_IMP, _("From _NoteCase File"), None, _("Add Nodes of a NoteCase File to the Current Tree"), inst.nodes_add_from_notecase_file),
+    ( "FromTomboy", STR_STOCK_CT_IMP, _("From T_omboy Folder"), None, _("Add Nodes of a Tomboy Folder to the Current Tree"), inst.nodes_add_from_tomboy_folder),
+    ( "FromTreepad", STR_STOCK_CT_IMP, _("From T_reepad Lite File"), None, _("Add Nodes of a Treepad Lite File to the Current Tree"), inst.nodes_add_from_treepad_file),
+    ( "FromTuxCards", STR_STOCK_CT_IMP, _("From _TuxCards File"), None, _("Add Nodes of a TuxCards File to the Current Tree"), inst.nodes_add_from_tuxcards_file),
+    ( "FromZim", STR_STOCK_CT_IMP, _("From _Zim Folder"), None, _("Add Nodes of a Zim Folder to the Current Tree"), inst.nodes_add_from_zim_folder),
     ( "Help", "help-contents", _("Online _Manual"), None, _("Application's Online Manual"), inst.on_help_menu_item_activated),
     ( "CheckNewer", "gtk-network", _("_Check Newer Version"), None, _("Check for a Newer Version"), inst.check_for_newer_version),
     ( "About", "gtk-about", _("_About"), None, _("About CherryTree"), inst.dialog_about),
@@ -668,9 +695,6 @@ def get_popup_menu_tree(inst):
     ("gtk-sort-descending", _("Sort Siblings D_escending"), None, _("Sort all the Siblings of the Selected Node Descending"), inst.node_siblings_sort_descending),
     ("submenu-end", None, None, None, None),
     ("separator", None, None, None, None),
-    ("gtk-zoom-in", _("E_xpand All Nodes"), None, _("Expand All the Tree Nodes"), inst.nodes_expand_all),
-    ("gtk-zoom-out", _("_Collapse All Nodes"), None, _("Collapse All the Tree Nodes"), inst.nodes_collapse_all),
-    ("separator", None, None, None, None),
     ("find", _("Find a _Node"), "<control>T", _("Find a Node from its Name"), inst.find_a_node),
     ("find_replace", _("Replace in Nodes _Names"), "<control><shift>T", _("Replace in Nodes Names"), inst.replace_in_nodes_names),
     ("separator", None, None, None, None),
@@ -679,6 +703,7 @@ def get_popup_menu_tree(inst):
     (STR_STOCK_CT_IMP, _("From _CherryTree File"), None, _("Add Nodes of a CherryTree File to the Current Tree"), inst.nodes_add_from_cherrytree_file),
     (STR_STOCK_CT_IMP, _("From _Gnote Folder"), None, _("Add Nodes of a Gnote Folder to the Current Tree"), inst.nodes_add_from_gnote_folder),
     (STR_STOCK_CT_IMP, _("From _KeepNote Folder"), None, _("Add Nodes of a KeepNote Folder to the Current Tree"), inst.nodes_add_from_keepnote_folder),
+    (STR_STOCK_CT_IMP, _("From K_eyNote File"), None, _("Add Nodes of a KeyNote File to the Current Tree"), inst.nodes_add_from_keynote_file),
     (STR_STOCK_CT_IMP, _("From K_nowit File"), None, _("Add Nodes of a Knowit File to the Current Tree"), inst.nodes_add_from_knowit_file),
     (STR_STOCK_CT_IMP, _("From _Leo File"), None, _("Add Nodes of a Leo File to the Current Tree"), inst.nodes_add_from_leo_file),
     (STR_STOCK_CT_IMP, _("From _Mempad File"), None, _("Add Nodes of a Mempad File to the Current Tree"), inst.nodes_add_from_mempad_file),
@@ -690,7 +715,8 @@ def get_popup_menu_tree(inst):
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("Nodes E_xport"), "export_from_cherrytree", None, None),
     ("gtk-print", _("_Print / Export To PDF"), "<control>P", _("Print / Export To PDF"), inst.export_print),
-    ("to_txt", _("Export to Plain _Text"), None, _("Export to Plain Text"), inst.export_to_txt),
+    ("to_txt", _("Export to Multiple Plain _Text Files"), None, _("Export to Multiple Plain Text Files"), inst.export_to_txt_multiple),
+    ("to_txt", _("Export to _Single Plain Text File"), None, _("Export to Single Plain Text File"), inst.export_to_txt_single),
     ("to_html", _("Export To _HTML"), None, _("Export To HTML"), inst.export_to_html),
     ("export_from_cherrytree", _("_Export To CherryTree Document"), None, _("Export To CherryTree Document"), inst.export_to_ctd),
     ("submenu-end", None, None, None, None),
@@ -715,7 +741,7 @@ def get_popup_menu_entries_text(inst):
     ("separator", None, None, None, None),
     ("submenu-start", _("For_matting") , "format_text", None, None),
     ("format_text_latest", _("Format _Latest"), "F7", _("Memory of Latest Text Format Type"), inst.apply_tag_latest),
-    ("gtk-clear", _("_Remove Formatting"), None, _("Remove the Formatting from the Selected Text"), inst.remove_text_formatting),
+    ("gtk-clear", _("_Remove Formatting"), "<control><shift>R", _("Remove the Formatting from the Selected Text"), inst.remove_text_formatting),
     ("separator", None, None, None, None),
     ("color_foreground", _("Text _Color Foreground"), None, _("Change the Color of the Selected Text Foreground"), inst.apply_tag_foreground),
     ("color_background", _("Text C_olor Background"), None, _("Change the Color of the Selected Text Background"), inst.apply_tag_background),
