@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -128,9 +128,9 @@ class ExportPrint:
 
     def pango_text_add_node_name(self, tree_iter, pango_text):
         """Add Node Name to Pango Text Vector"""
-        pango_text[0] = "<span size=\"xx-large\">" \
-                      + self.dad.treestore[tree_iter][1] \
-                      + "</span>" + 2*cons.CHAR_NEWLINE + pango_text[0]
+        pango_text[0] = "<b><i><span size=\"xx-large\">" \
+                      + cgi.escape(self.dad.treestore[tree_iter][1]) \
+                      + "</span></i></b>" + 2*cons.CHAR_NEWLINE + pango_text[0]
 
     def node_export_print(self, tree_iter, include_node_name, only_selection):
         """Export Print the Selected Node"""
@@ -180,46 +180,106 @@ class Export2Txt:
         """Prepare the website folder"""
         dir_place = support.dialog_folder_select(curr_folder=self.dad.file_dir, parent=self.dad.window)
         if dir_place == None: return False
-        new_folder += "_txt"
+        new_folder = support.clean_from_chars_not_for_filename(new_folder) + "_TXT"
         while os.path.exists(os.path.join(dir_place, new_folder)):
             new_folder += "2"
         self.new_path = os.path.join(dir_place, new_folder)
         os.mkdir(self.new_path)
         return True
 
-    def nodes_all_export_to_txt(self, top_tree_iter=None):
+    def nodes_all_export_to_txt(self, top_tree_iter=None, single_txt_filepath="", include_node_name=True):
         """Export All Nodes To Txt"""
         if not top_tree_iter: tree_iter = self.dad.treestore.get_iter_first()
         else: tree_iter = top_tree_iter.copy()
         while tree_iter:
-            self.nodes_all_export_to_txt_iter(tree_iter)
+            self.nodes_all_export_to_txt_iter(tree_iter, single_txt_filepath, include_node_name)
             if top_tree_iter: break
             tree_iter = self.dad.treestore.iter_next(tree_iter)
         self.dad.objects_buffer_refresh()
 
-    def nodes_all_export_to_txt_iter(self, tree_iter):
+    def nodes_all_export_to_txt_iter(self, tree_iter, single_txt_filepath, include_node_name):
         """Export All Nodes To Txt - iter"""
         text_buffer = self.dad.get_textbuffer_from_tree_iter(tree_iter)
-        filepath = os.path.join(self.new_path,
-                                support.get_node_hierarchical_name(self.dad, tree_iter) + ".txt")
-        self.node_export_to_txt(text_buffer, filepath)
+        tree_iter_for_node_name = tree_iter if include_node_name else None
+        if not single_txt_filepath:
+            filepath = os.path.join(self.new_path,
+                                    support.get_node_hierarchical_name(self.dad, tree_iter) + ".txt")
+            self.node_export_to_txt(text_buffer, filepath, tree_iter_for_node_name=tree_iter_for_node_name)
+        else:
+            self.node_export_to_txt(text_buffer, single_txt_filepath, tree_iter_for_node_name=tree_iter_for_node_name)
         child_tree_iter = self.dad.treestore.iter_children(tree_iter)
         while child_tree_iter:
-            self.nodes_all_export_to_txt_iter(child_tree_iter)
+            self.nodes_all_export_to_txt_iter(child_tree_iter, single_txt_filepath, include_node_name)
             child_tree_iter = self.dad.treestore.iter_next(child_tree_iter)
 
-    def node_export_to_txt(self, text_buffer, filepath, sel_range=None):
-        """Export the Selected Node To Txt"""
-        if sel_range:
-            iter_start = text_buffer.get_iter_at_offset(sel_range[0])
-            iter_end = text_buffer.get_iter_at_offset(sel_range[1])
-        else:
-            iter_start, iter_end = text_buffer.get_bounds()
-        plain_text = text_buffer.get_text(iter_start, iter_end)
-        file_descriptor = open(filepath, 'w')
-        file_descriptor.write(plain_text)
-        file_descriptor.close()
+    def plain_process_slot(self, start_offset, end_offset, curr_buffer):
+        """Process a Single plain Slot"""
+        start_iter = curr_buffer.get_iter_at_offset(start_offset)
+        if end_offset == -1:
+            end_offset = curr_buffer.get_end_iter().get_offset()
+        end_iter = curr_buffer.get_iter_at_offset(end_offset)
+        #print "process slot (%s->%s)" % (start_offset, end_offset)
+        # begin operations
+        self.curr_plain_slots.append(curr_buffer.get_text(start_iter, end_iter))
 
+    def get_codebox_plain(self, codebox):
+        """Returns the plain CodeBox"""
+        # codebox is: [offset, dict, justification]
+        codebox_plain = cons.CHAR_NEWLINE + self.dad.h_rule + cons.CHAR_NEWLINE
+        codebox_plain += codebox[1]['fill_text']
+        codebox_plain += cons.CHAR_NEWLINE + self.dad.h_rule + cons.CHAR_NEWLINE
+        return codebox_plain
+
+    def get_table_plain(self, table_orig):
+        """Returns the plain Table"""
+        # table is: [offset, dict, justification]
+        table_plain = cons.CHAR_NEWLINE + self.dad.h_rule + cons.CHAR_NEWLINE + cons.CHAR_PIPE*2
+        table = copy.deepcopy(table_orig)
+        table[1]['matrix'].insert(0, table[1]['matrix'].pop())
+        for j, row in enumerate(table[1]['matrix']):
+            for cell in row:
+                table_plain += cons.CHAR_SPACE + cell + cons.CHAR_SPACE + cons.CHAR_PIPE*2
+            table_plain += cons.CHAR_NEWLINE + self.dad.h_rule + cons.CHAR_NEWLINE
+        return table_plain
+
+    def plain_get_from_treestore_node(self, curr_buffer, sel_range=None):
+        """Given a treestore iter returns the HTML rich text"""
+        pixbuf_table_codebox_vector = self.dad.state_machine.get_embedded_pixbufs_tables_codeboxes(curr_buffer,
+                                                                                                   for_print=2,
+                                                                                                   sel_range=sel_range)
+        # pixbuf_table_codebox_vector is [ [ "pixbuf"/"table"/"codebox", [offset, pixbuf, alignment] ],... ]
+        self.curr_plain_slots = []
+        start_offset = 0 if not sel_range else sel_range[0]
+        for end_offset_element in pixbuf_table_codebox_vector:
+            end_offset = end_offset_element[1][0]
+            self.plain_process_slot(start_offset, end_offset, curr_buffer)
+            start_offset = end_offset
+        if not sel_range: self.plain_process_slot(start_offset, -1, curr_buffer)
+        else: self.plain_process_slot(start_offset, sel_range[1], curr_buffer)
+        #print "curr_plain_slots", curr_plain_slots
+        #print "pixbuf_table_codebox_vector", pixbuf_table_codebox_vector
+        return [self.curr_plain_slots, pixbuf_table_codebox_vector]
+
+    def node_export_to_txt(self, text_buffer, filepath, sel_range=None, tree_iter_for_node_name=None):
+        """Export the Selected Node To Txt"""
+        plain_text = ""
+        text_n_objects = self.plain_get_from_treestore_node(text_buffer, sel_range)
+        self.images_count = 0
+        for i, plain_slot in enumerate(text_n_objects[0]):
+            plain_text += plain_slot
+            if i < len(text_n_objects[1]):
+                curr_object = text_n_objects[1][i]
+                if curr_object[0] == "table": plain_text += self.get_table_plain(curr_object[1])
+                elif curr_object[0] == "codebox": plain_text += self.get_codebox_plain(curr_object[1])
+        if tree_iter_for_node_name:
+            plain_text = self.plain_text_get_node_name(tree_iter_for_node_name) + plain_text
+        file_descriptor = open(filepath, 'a')
+        file_descriptor.write(plain_text + 2*cons.CHAR_NEWLINE)
+        file_descriptor.close()
+    
+    def plain_text_get_node_name(self, tree_iter):
+        """Get Node Name in Plain Text"""
+        return self.dad.treestore[tree_iter][1].upper() + cons.CHAR_NEWLINE
 
 class Export2Pango:
     """The Export to Pango Class"""
@@ -371,7 +431,7 @@ class Export2Html:
         """Prepare the website folder"""
         dir_place = support.dialog_folder_select(curr_folder=self.dad.file_dir, parent=self.dad.window)
         if dir_place == None: return False
-        new_folder += "_html"
+        new_folder = support.clean_from_chars_not_for_filename(new_folder) + "_HTML"
         while os.path.exists(os.path.join(dir_place, new_folder)):
             new_folder += "2"
         self.new_path = os.path.join(dir_place, new_folder)
@@ -471,7 +531,9 @@ class Export2Html:
         if self.tree_links_text:
             html_text += '</td></tr></table>'
         html_text += cons.HTML_FOOTER
-        file_descriptor = open(os.path.join(self.new_path, self.get_html_filename(tree_iter)), 'w')
+        node_html_filepath = os.path.join(self.new_path, self.get_html_filename(tree_iter))
+        #print "full=%s(prefix=%s)" % (len(node_html_filepath), len(self.new_path))
+        file_descriptor = open(node_html_filepath, 'w')
         file_descriptor.write(html_text)
         file_descriptor.close()
 
@@ -561,13 +623,15 @@ class Export2Html:
 
     def get_html_filename(self, tree_iter):
         """Get the HTML page filename given the tree iter"""
-        file_name = self.dad.treestore[tree_iter][1]
+        file_name = self.dad.treestore[tree_iter][1].strip()
         father_iter = self.dad.treestore.iter_parent(tree_iter)
         while father_iter:
-            file_name = self.dad.treestore[father_iter][1] + "--" + file_name
+            file_name = self.dad.treestore[father_iter][1].strip() + "--" + file_name
             father_iter = self.dad.treestore.iter_parent(father_iter)
-        file_name += ".html"
-        return file_name.replace("/", "-")
+        file_name = support.clean_from_chars_not_for_filename(file_name) + ".html"
+        if len(file_name) > cons.MAX_FILE_NAME_LEN:
+            file_name = file_name[-cons.MAX_FILE_NAME_LEN:]
+        return file_name
 
     def html_get_from_code_buffer(self, code_buffer, sel_range=None):
         """Get rich text from syntax highlighted code node"""
@@ -742,9 +806,10 @@ class Export2Html:
                 elif tag_property == cons.TAG_LINK:
                     # <a href="http://www.example.com/">link-text goes here</a>
                     vector = property_value.split()
-                    if vector[0] == "webs": href = vector[1]
-                    elif vector[0] == "file": href = "file://" + base64.b64decode(vector[1])
-                    elif vector[0] == "node":
+                    if vector[0] == cons.LINK_TYPE_WEBS: href = vector[1]
+                    elif vector[0] == cons.LINK_TYPE_FILE:
+                        href = "file://" + base64.b64decode(vector[1])
+                    elif vector[0] == cons.LINK_TYPE_NODE:
                         dest_tree_iter = self.dad.get_tree_iter_from_node_id(long(vector[1]))
                         if not dest_tree_iter: continue
                         href = self.get_html_filename(dest_tree_iter)
