@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -73,13 +73,11 @@ class FindReplace:
             iter_bound = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_selection_bound())
             entry_predefined_text = self.dad.curr_buffer.get_text(iter_insert, iter_bound)
             if not self.replace_active:
-                pattern = self.dad.dialog_input(title=_("Search in Current Node..."),
-                                            entry_hint=entry_predefined_text,
-                                            search_opt=True)
+                pattern = self.dad.dialog_search(title=_("Search in Current Node..."),
+                                            entry_hint=entry_predefined_text)
             else:
-                pattern = self.dad.dialog_input(title=_("Replace in Current Node..."),
-                                            entry_hint=entry_predefined_text,
-                                            search_opt=True, replace_opt=True)
+                pattern = self.dad.dialog_search(title=_("Replace in Current Node..."),
+                                            entry_hint=entry_predefined_text, replace_opt=True)
             if entry_predefined_text != "":
                 self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_insert(), iter_insert)
                 self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_selection_bound(), iter_bound)
@@ -122,13 +120,11 @@ class FindReplace:
             iter_bound = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_selection_bound())
             entry_predefined_text = self.dad.curr_buffer.get_text(iter_insert, iter_bound)
             if not self.replace_active:
-                pattern = self.dad.dialog_input(title=_("Search in All Nodes..."),
-                                            entry_hint=entry_predefined_text,
-                                            search_opt=True)
+                pattern = self.dad.dialog_search(title=_("Search in All Nodes..."),
+                                            entry_hint=entry_predefined_text)
             else:
-                pattern = self.dad.dialog_input(title=_("Replace in All Nodes..."),
-                                            entry_hint=entry_predefined_text,
-                                            search_opt=True, replace_opt=True)
+                pattern = self.dad.dialog_search(title=_("Replace in All Nodes..."),
+                                            entry_hint=entry_predefined_text, replace_opt=True)
             if entry_predefined_text != "":
                 self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_insert(), iter_insert)
                 self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_selection_bound(), iter_bound)
@@ -176,15 +172,16 @@ class FindReplace:
                     if forward: node_iter = self.dad.treestore.iter_next(node_iter)
                     else: node_iter = self.dad.get_tree_iter_prev_sibling(self.dad.treestore, node_iter)
                 else: break
-        if self.matches_num == 0:
-            config.set_tree_expanded_collapsed_string(self.dad)
-            support.dialog_info(_("The pattern '%s' was not found") % pattern, self.dad.window)
+        if user_active_restore: self.dad.user_active = True
+        config.set_tree_expanded_collapsed_string(self.dad)
+        if not self.matches_num or all_matches:
             self.dad.treeview_safe_set_cursor(starting_tree_iter)
             self.dad.sourceview.grab_focus()
             self.dad.curr_buffer.place_cursor(self.dad.curr_buffer.get_iter_at_offset(current_cursor_pos))
             self.dad.sourceview.scroll_to_mark(self.dad.curr_buffer.get_insert(), 0.3)
+        if not self.matches_num:
+            support.dialog_info(_("The pattern '%s' was not found") % pattern, self.dad.window)
         else:
-            config.set_tree_expanded_collapsed_string(self.dad)
             if all_matches:
                 self.dad.glade.allmatchesdialog.set_title(str(self.matches_num) + cons.CHAR_SPACE + _("Matches"))
                 self.dad.glade.allmatchesdialog.run()
@@ -195,15 +192,14 @@ class FindReplace:
                 self.dad.sourceview.scroll_to_mark(self.dad.curr_buffer.get_insert(), 0.3)
                 if self.dad.glade.checkbutton_iterated_find_dialog.get_active():
                     self.iterated_find_dialog()
-        if user_active_restore: self.dad.user_active = True
 
     def find_a_node(self, *args):
         """Search for a pattern between all the Node's Names"""
         if not self.from_find_iterated:
             if not self.replace_active:
-                pattern_clean = self.dad.dialog_input(title=_("Search For a Node Name..."), search_opt=True)
+                pattern_clean = self.dad.dialog_search(title=_("Search For a Node Name..."))
             else:
-                pattern_clean = self.dad.dialog_input(title=_("Replace in Node Names..."), search_opt=True, replace_opt=True)
+                pattern_clean = self.dad.dialog_search(title=_("Replace in Node Names..."), replace_opt=True)
             if pattern_clean != None: self.curr_find = ["a_node", pattern_clean]
             else: return
         else: pattern_clean = self.curr_find[1]
@@ -267,7 +263,7 @@ class FindReplace:
 
     def find_pattern(self, pattern, start_iter, forward, all_matches):
         """Returns (start_iter, end_iter) or (None, None)"""
-        text = self.dad.curr_buffer.get_text(*self.dad.curr_buffer.get_bounds()).decode("utf-8")
+        text = unicode(self.dad.curr_buffer.get_text(*self.dad.curr_buffer.get_bounds()), cons.STR_UTF8, cons.STR_IGNORE)
         if not self.dad.glade.checkbutton_re.get_active(): # NOT REGULAR EXPRESSION
             pattern = re.escape(pattern) # backslashes all non alphanum chars => to not spoil re
             if self.dad.glade.checkbutton_whole_word.get_active(): # WHOLE WORD
@@ -322,7 +318,7 @@ class FindReplace:
                                        self.dad.treestore[self.dad.curr_tree_iter][1],
                                        self.get_line_content(iter_insert) if obj_match_offsets[0] == None else obj_match_offsets[2]])
             if self.replace_active:
-                replacer_text = self.dad.glade.replace_entry.get_text().decode("utf-8")
+                replacer_text = unicode(self.dad.glade.replace_entry.get_text(), cons.STR_UTF8, cons.STR_IGNORE)
                 self.dad.curr_buffer.delete_selection(interactive=False, default_editable=True)
                 self.dad.curr_buffer.insert_at_cursor(replacer_text)
                 if not all_matches:
@@ -449,10 +445,10 @@ class FindReplace:
 
     def parse_node_name(self, node_iter, pattern, forward, all_matches):
         """Recursive function that searchs for the given pattern"""
-        text_name = self.dad.treestore[node_iter][1].decode("utf-8")
+        text_name = self.dad.treestore[node_iter][1].decode(cons.STR_UTF8)
         match = pattern.search(text_name)
         if not match:
-            text_tags = self.dad.treestore[node_iter][6].decode("utf-8")
+            text_tags = self.dad.treestore[node_iter][6].decode(cons.STR_UTF8)
             match = pattern.search(text_tags)
         if match:
             if all_matches:
@@ -462,7 +458,7 @@ class FindReplace:
                                        self.dad.treestore[node_iter][1],
                                        self.get_first_line_content(self.dad.get_textbuffer_from_tree_iter(node_iter))])
             if self.replace_active:
-                replacer_text = self.dad.glade.replace_entry.get_text().decode("utf-8")
+                replacer_text = unicode(self.dad.glade.replace_entry.get_text(), cons.STR_UTF8, cons.STR_IGNORE)
                 text_name = text_name.replace(self.curr_find[1], replacer_text)
                 self.dad.treestore[node_iter][1] = text_name
                 self.ctdb_handler.pending_edit_db_node_prop(self.dad.treestore[node_iter][3])
