@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -320,6 +320,10 @@ class XMLHandler:
             self.rich_text_attributes_update(curr_iter, self.curr_attributes)
             tag_found = curr_iter.forward_to_tag_toggle(None)
             while tag_found:
+                if not self.tag_richtext_toggling_on_or_off(curr_iter):
+                    if not curr_iter.forward_char(): tag_found = False
+                    else: tag_found = curr_iter.forward_to_tag_toggle(None)
+                    continue
                 self.rich_txt_serialize(dom_iter, start_iter, curr_iter, self.curr_attributes)
                 if curr_iter.compare(end_iter) >= 0: break
                 else:
@@ -409,99 +413,144 @@ class XMLHandler:
         text_iter = dom.createTextNode(slot_text)
         dom_iter.appendChild(text_iter)
 
+    def tag_header_toggling_on_or_off(self, curr_iter):
+        """Check for tag header toggle on or off"""
+        toggled_onoff = []
+        toggled_off = curr_iter.get_toggled_tags(toggled_on=False)
+        toggled_on = curr_iter.get_toggled_tags(toggled_on=True)
+        if toggled_off: toggled_onoff.extend(toggled_off)
+        if toggled_on: toggled_onoff.extend(toggled_on)
+        for tag in toggled_onoff:
+            tag_name = tag.get_property("name")
+            if tag_name and tag_name.startswith("scale_"): return True
+        return False
+        
+    def tag_richtext_toggling_on_or_off(self, curr_iter):
+        """Check for tag rich text toggle on or off"""
+        toggled_onoff = []
+        toggled_off = curr_iter.get_toggled_tags(toggled_on=False)
+        toggled_on = curr_iter.get_toggled_tags(toggled_on=True)
+        if toggled_off: toggled_onoff.extend(toggled_off)
+        if toggled_on: toggled_onoff.extend(toggled_on)
+        for tag in toggled_onoff:
+            tag_name = tag.get_property("name")
+            if not tag_name: continue
+            if tag_name.startswith("weight_") \
+            or tag_name.startswith("foreground_") \
+            or tag_name.startswith("background_") \
+            or tag_name.startswith("style_") \
+            or tag_name.startswith("underline_") \
+            or tag_name.startswith("strikethrough_") \
+            or tag_name.startswith("scale_") \
+            or tag_name.startswith("justification_") \
+            or tag_name.startswith("link_") \
+            or tag_name.startswith("family_"):
+                return True
+        return False
+
     def rich_text_attributes_update(self, curr_iter, curr_attributes):
         """Updates the list of Attributes for the Current Slice"""
         toggled_off = curr_iter.get_toggled_tags(toggled_on=False)
         for tag in toggled_off:
             tag_name = tag.get_property("name")
-            if tag_name:
-                if tag_name[0:7] == "weight_": curr_attributes[cons.TAG_WEIGHT] = ""
-                elif tag_name[0:11] == "foreground_": curr_attributes[cons.TAG_FOREGROUND] = ""
-                elif tag_name[0:11] == "background_": curr_attributes[cons.TAG_BACKGROUND] = ""
-                elif tag_name[0:6] == "style_": curr_attributes[cons.TAG_STYLE] = ""
-                elif tag_name[0:10] == "underline_": curr_attributes[cons.TAG_UNDERLINE] = ""
-                elif tag_name[0:14] == "strikethrough_": curr_attributes[cons.TAG_STRIKETHROUGH] = ""
-                elif tag_name[0:6] == "scale_": curr_attributes[cons.TAG_SCALE] = ""
-                elif tag_name[0:14] == "justification_": curr_attributes[cons.TAG_JUSTIFICATION] = ""
-                elif tag_name[0:5] == "link_": curr_attributes[cons.TAG_LINK] = ""
-                elif tag_name[0:7] == "family_": curr_attributes[cons.TAG_FAMILY] = ""
+            if tag_name and tag_name != cons.GTKSPELLCHECK_TAG_NAME:
+                if tag_name.startswith("weight_"): curr_attributes[cons.TAG_WEIGHT] = ""
+                elif tag_name.startswith("foreground_"): curr_attributes[cons.TAG_FOREGROUND] = ""
+                elif tag_name.startswith("background_"): curr_attributes[cons.TAG_BACKGROUND] = ""
+                elif tag_name.startswith("style_"): curr_attributes[cons.TAG_STYLE] = ""
+                elif tag_name.startswith("underline_"): curr_attributes[cons.TAG_UNDERLINE] = ""
+                elif tag_name.startswith("strikethrough_"): curr_attributes[cons.TAG_STRIKETHROUGH] = ""
+                elif tag_name.startswith("scale_"): curr_attributes[cons.TAG_SCALE] = ""
+                elif tag_name.startswith("justification_"): curr_attributes[cons.TAG_JUSTIFICATION] = ""
+                elif tag_name.startswith("link_"): curr_attributes[cons.TAG_LINK] = ""
+                elif tag_name.startswith("family_"): curr_attributes[cons.TAG_FAMILY] = ""
                 else: support.dialog_error("Failure processing the toggling OFF tag %s" % tag_name, self.dad.window)
         toggled_on = curr_iter.get_toggled_tags(toggled_on=True)
         for tag in toggled_on:
             tag_name = tag.get_property("name")
-            if tag_name:
-                if tag_name[0:7] == "weight_": curr_attributes[cons.TAG_WEIGHT] = tag_name[7:]
-                elif tag_name[0:11] == "foreground_":
+            if tag_name and tag_name != cons.GTKSPELLCHECK_TAG_NAME:
+                if tag_name.startswith("weight_"): curr_attributes[cons.TAG_WEIGHT] = tag_name[7:]
+                elif tag_name.startswith("foreground_"):
                     new_foreground = tag_name[11:]
                     if new_foreground != cons.COLOR_48_BLACK: curr_attributes[cons.TAG_FOREGROUND] = new_foreground
-                elif tag_name[0:11] == "background_":
+                elif tag_name.startswith("background_"):
                     new_background = tag_name[11:]
                     if new_background != cons.COLOR_48_WHITE: curr_attributes[cons.TAG_BACKGROUND] = new_background
-                elif tag_name[0:6] == "scale_": curr_attributes[cons.TAG_SCALE] = tag_name[6:]
-                elif tag_name[0:14] == "justification_": curr_attributes[cons.TAG_JUSTIFICATION] = tag_name[14:]
-                elif tag_name[0:6] == "style_": curr_attributes[cons.TAG_STYLE] = tag_name[6:]
-                elif tag_name[0:10] == "underline_": curr_attributes[cons.TAG_UNDERLINE] = tag_name[10:]
-                elif tag_name[0:14] == "strikethrough_": curr_attributes[cons.TAG_STRIKETHROUGH] = tag_name[14:]
-                elif tag_name[0:5] == "link_": curr_attributes[cons.TAG_LINK] = tag_name[5:]
-                elif tag_name[0:7] == "family_": curr_attributes[cons.TAG_FAMILY] = tag_name[7:]
+                elif tag_name.startswith("scale_"): curr_attributes[cons.TAG_SCALE] = tag_name[6:]
+                elif tag_name.startswith("justification_"): curr_attributes[cons.TAG_JUSTIFICATION] = tag_name[14:]
+                elif tag_name.startswith("style_"): curr_attributes[cons.TAG_STYLE] = tag_name[6:]
+                elif tag_name.startswith("underline_"): curr_attributes[cons.TAG_UNDERLINE] = tag_name[10:]
+                elif tag_name.startswith("strikethrough_"): curr_attributes[cons.TAG_STRIKETHROUGH] = tag_name[14:]
+                elif tag_name.startswith("link_"): curr_attributes[cons.TAG_LINK] = tag_name[5:]
+                elif tag_name.startswith("family_"): curr_attributes[cons.TAG_FAMILY] = tag_name[7:]
                 else: support.dialog_error("Failure processing the toggling ON tag %s" % tag_name, self.dad.window)
 
     def toc_insert_all(self, text_buffer, top_tree_iter):
         """Insert a TOC at top of text_buffer, including Node and Subnodes starting from top_tree_iter"""
         config.get_tree_expanded_collapsed_string(self.dad)
         starting_tree_iter = self.dad.curr_tree_iter.copy()
-        toc_list_per_node = []
-        while top_tree_iter != None:
-            toc_list_per_node.extend(self.toc_insert_all_iter(top_tree_iter))
-            top_tree_iter = self.dad.treestore.iter_next(top_tree_iter)
+        if top_tree_iter: toc_list_per_node = self.toc_insert_all_iter(top_tree_iter)
+        else:
+            top_tree_iter = self.dad.treestore.get_iter_first()
+            toc_list_per_node = []
+            while top_tree_iter != None:
+                toc_list_per_node.extend(self.toc_insert_all_iter(top_tree_iter))
+                top_tree_iter = self.dad.treestore.iter_next(top_tree_iter)
         config.set_tree_expanded_collapsed_string(self.dad)
         self.dad.treeview_safe_set_cursor(starting_tree_iter)
         self.dad.objects_buffer_refresh()
         #print toc_list_per_node
         curr_node_id = -1
+        curr_node_level = 0
         if toc_list_per_node:
             tag_property = cons.TAG_LINK
             curr_offset = 0
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
             curr_offset += 1
             for element in toc_list_per_node:
-                property_value = "node" + cons.CHAR_SPACE + str(element[2])
+                property_value = cons.LINK_TYPE_NODE + cons.CHAR_SPACE + str(element[2])
                 if curr_node_id != element[2]:
                     curr_node_id = element[2]
                     node_tree_iter = self.dad.get_tree_iter_from_node_id(curr_node_id)
-                    node_name = self.dad.treestore[node_tree_iter][1]
+                    curr_node_level = self.dad.treestore.iter_depth(node_tree_iter)
+                    node_name = self.dad.treestore[node_tree_iter][1].replace(cons.CHAR_NEWLINE, cons.CHAR_SPACE).replace(cons.CHAR_CR, "")
                     tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value)]
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 2
-                    text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), node_name, *tag_names)
-                    curr_offset += len(node_name)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*curr_node_level*cons.CHAR_SPACE + cons.CHAR_LISTARR + cons.CHAR_SPACE)
+                    curr_offset += 2 + 2*curr_node_level
                     text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
+                    text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), node_name, *tag_names)
+                    curr_offset += 1
+                    while text_buffer.get_iter_at_offset(curr_offset).get_char() != cons.CHAR_NEWLINE:
+                        curr_offset += 1
                     curr_offset += 1
                 tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0])]
+                if not element[0]: continue
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 3*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 5
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    curr_offset += 4 + 2*curr_node_level
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 6*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 8
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    curr_offset += 6 + 2*curr_node_level
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 9*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 11
-                text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
-                curr_offset += len(element[1])
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    curr_offset += 8 + 2*curr_node_level
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
+                text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
+                curr_offset += 1
+                while text_buffer.get_iter_at_offset(curr_offset).get_char() != cons.CHAR_NEWLINE:
+                    curr_offset += 1
                 curr_offset += 1
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
         return toc_list_per_node
         
     def toc_insert_all_iter(self, top_tree_iter):
         """Iterate on nodes for toc_insert_all"""
-        if self.dad.treestore[top_tree_iter][4] != cons.CUSTOM_COLORS_ID: return []
         self.dad.treeview_safe_set_cursor(top_tree_iter)
         node_id = self.dad.treestore[top_tree_iter][3]
         text_buffer = self.dad.get_textbuffer_from_tree_iter(top_tree_iter)
         toc_list_per_node = []
-        toc_list_per_node.extend(self.toc_insert_one(text_buffer, node_id, just_get_toc_list=True))
+        toc_list_this_one = self.toc_insert_one(text_buffer, node_id, just_get_toc_list=True)
+        toc_list_per_node.extend(toc_list_this_one or [["", "", node_id]])
         child_tree_iter = self.dad.treestore.iter_children(top_tree_iter)
         while child_tree_iter != None:
             toc_list_per_node.extend(self.toc_insert_all_iter(child_tree_iter))
@@ -520,11 +569,14 @@ class XMLHandler:
         self.rich_text_attributes_update(curr_iter, self.curr_attributes)
         tag_found = curr_iter.forward_to_tag_toggle(None)
         while tag_found:
+            if not self.tag_header_toggling_on_or_off(curr_iter):
+                if not curr_iter.forward_char(): tag_found = False
+                else: tag_found = curr_iter.forward_to_tag_toggle(None)
+                continue
             offsets = self.toc_insert_parser(text_buffer, start_iter, curr_iter, node_id)
             if offsets:
                 start_iter = text_buffer.get_iter_at_offset(offsets[0])
                 curr_iter = text_buffer.get_iter_at_offset(offsets[1])
-                end_iter = text_buffer.get_end_iter()
             if curr_iter.compare(end_iter) == 0: break
             else:
                 self.rich_text_attributes_update(curr_iter, self.curr_attributes)
@@ -535,7 +587,7 @@ class XMLHandler:
         else: self.toc_insert_parser(text_buffer, start_iter, curr_iter, node_id)
         if self.toc_list and not just_get_toc_list:
             tag_property = cons.TAG_LINK
-            property_value = "node" + cons.CHAR_SPACE + str(node_id)
+            property_value = cons.LINK_TYPE_NODE + cons.CHAR_SPACE + str(node_id)
             curr_offset = 0
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
             curr_offset += 1
@@ -546,14 +598,16 @@ class XMLHandler:
                     text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_LISTBUL + cons.CHAR_SPACE)
                     curr_offset += 2
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 3*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 5
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    curr_offset += 4
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 6*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    curr_offset += 8
-                text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
-                curr_offset += len(element[1])
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    curr_offset += 6
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
+                text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
+                curr_offset += 1
+                while text_buffer.get_iter_at_offset(curr_offset).get_char() != cons.CHAR_NEWLINE:
+                    curr_offset += 1
                 curr_offset += 1
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
         return self.toc_list
@@ -584,6 +638,7 @@ class XMLHandler:
         pixbuf = pixbuf.scale_simple(self.dad.anchor_size, self.dad.anchor_size, gtk.gdk.INTERP_BILINEAR)
         pixbuf.anchor = self.toc_list[-1][0]
         self.dad.image_insert(start_iter, pixbuf)
+        self.dad.ctdb_handler.pending_edit_db_node_buff(node_id, force_user_active=True)
         return (start_offset+1, end_offset+1)
 
 
@@ -612,9 +667,10 @@ class StateMachine:
             if anchor != None:
                 anchor_dir = dir(anchor)
                 if "pixbuf" in anchor_dir:
-                    pixbuf_table_codebox_vector.append(["pixbuf", [curr_iter.get_offset(),
-                                                        anchor.pixbuf,
-                                                        self.get_iter_alignment(curr_iter)] ])
+                    if for_print != 1 or not "anchor" in dir(anchor.pixbuf):
+                        pixbuf_table_codebox_vector.append(["pixbuf", [curr_iter.get_offset(),
+                                                            anchor.pixbuf,
+                                                            self.get_iter_alignment(curr_iter)] ])
                 elif "liststore" in anchor_dir:
                     pixbuf_table_codebox_vector.append(["table", [curr_iter.get_offset(),
                                                         self.table_to_dict(anchor),
