@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -128,7 +128,7 @@ class CherryTreeHandler():
                     just_run_new_win = True
                     if not self.msg_server_to_core['p']:
                         for i, runn_win in enumerate(self.running_windows):
-                            if not runn_win.window.get_visible():
+                            if not runn_win.window.get_property(cons.STR_VISIBLE):
                                 print "3 rise existing hidden in systray"
                                 runn_win.window.present()
                                 just_run_new_win = False
@@ -147,7 +147,7 @@ def initializations():
     if not os.path.isdir(cons.CONFIG_DIR): os.mkdir(cons.CONFIG_DIR)
     if cons.IS_WIN_OS:
         import warnings
-        warnings.filterwarnings("ignore")
+        warnings.filterwarnings(cons.STR_IGNORE)
     else:
         try:
             # change process name
@@ -161,13 +161,13 @@ def initializations():
     try:
         # change locale text domain
         locale.bindtextdomain(cons.APP_NAME, cons.LOCALE_PATH)
-        locale.bind_textdomain_codeset(cons.APP_NAME, 'UTF-8')
+        locale.bind_textdomain_codeset(cons.APP_NAME, cons.STR_UTF8)
     except:
         try:
             from ctypes import cdll
             libintl = cdll.intl
             libintl.bindtextdomain(cons.APP_NAME, cons.LOCALE_PATH)
-            libintl.bind_textdomain_codeset(cons.APP_NAME, 'UTF-8')
+            libintl.bind_textdomain_codeset(cons.APP_NAME, cons.STR_UTF8)
         except:
             print "ctypes.cdll.intl.bindtextdomain not available, the glade i18n will not work properly"
         try:
