@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -23,6 +23,8 @@ import gtk, gobject, pango, cairo
 import copy
 import support, cons
 
+BOX_OFFSET = 4
+
 
 class PrintData:
     """Print Operation Data"""
@@ -143,6 +145,27 @@ class PrintHandler:
                 if i < len(print_data.layout) - 1: # the latest element is supposed to be text
                     if self.pixbuf_table_codebox_vector[i][0] == "pixbuf":
                         pixbuf = self.pixbuf_table_codebox_vector[i][1][1]
+                        pixbuf_was_resized = False
+                        pixbuf_width = pixbuf.get_width()
+                        pixbuf_height = pixbuf.get_height()
+                        if pixbuf_width > self.page_width:
+                            image_w_h_ration = float(pixbuf_width)/pixbuf_height
+                            image_width = self.page_width
+                            image_height = image_width / image_w_h_ration
+                            pixbuf = pixbuf.scale_simple(int(image_width),
+                                                         int(image_height),
+                                                         gtk.gdk.INTERP_BILINEAR)
+                            pixbuf_was_resized = True
+                        if pixbuf_height > self.page_height:
+                            image_w_h_ration = float(pixbuf_width)/pixbuf_height
+                            image_height = self.page_height
+                            image_width = image_height * image_w_h_ration
+                            pixbuf = pixbuf.scale_simple(int(image_width),
+                                                         int(image_height),
+                                                         gtk.gdk.INTERP_BILINEAR)
+                            pixbuf_was_resized = True
+                        if pixbuf_was_resized:
+                            self.pixbuf_table_codebox_vector[i][1][1] = pixbuf
                         pixbuf_height = pixbuf.get_height() + cons.WHITE_SPACE_BETW_PIXB_AND_TEXT
                         if inline_pending_height < pixbuf_height: inline_pending_height = pixbuf_height
                     elif self.pixbuf_table_codebox_vector[i][0] == "table":
@@ -151,12 +174,12 @@ class PrintHandler:
                         table_layouts = self.get_table_layouts(context, table)
                         table_grid = self.get_table_grid(table_layouts, table['col_min'])
                         table_height = self.get_table_height_from_grid(table_grid)
-                        if inline_pending_height < table_height: inline_pending_height = table_height
+                        if inline_pending_height < table_height+BOX_OFFSET: inline_pending_height = table_height+BOX_OFFSET
                     elif self.pixbuf_table_codebox_vector[i][0] == "codebox":
                         codebox_dict = self.pixbuf_table_codebox_vector[i][1][1]
                         codebox_layout = self.get_codebox_layout(context, codebox_dict)
                         codebox_height = self.get_height_from_layout(codebox_layout)
-                        if inline_pending_height < codebox_height: inline_pending_height = codebox_height
+                        if inline_pending_height < codebox_height+BOX_OFFSET: inline_pending_height = codebox_height+BOX_OFFSET
             if exit_ok: break
         operation.set_n_pages(len(print_data.page_breaks) + 1)
 
