@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -39,13 +39,25 @@ def text_file_rm_emptylines(filepath):
         fd.writelines(file_lines)
         fd.close()
 
+def clean_from_chars_not_for_filename(filename_in):
+    """Clean a string from chars not good for filename"""
+    filename_out = filename_in.replace(cons.CHAR_SLASH, cons.CHAR_MINUS).replace(cons.CHAR_BSLASH, cons.CHAR_MINUS)
+    filename_out = filename_out.replace(cons.CHAR_STAR, "").replace(cons.CHAR_QUESTION, "").replace(cons.CHAR_COLON, "")
+    filename_out = filename_out.replace(cons.CHAR_LESSER, "").replace(cons.CHAR_GREATER, "")
+    filename_out = filename_out.replace(cons.CHAR_PIPE, "").replace(cons.CHAR_DQUOTE, "")
+    filename_out = filename_out.replace(cons.CHAR_NEWLINE, "").replace(cons.CHAR_CR, "").strip()
+    return filename_out.replace(cons.CHAR_SPACE, cons.CHAR_USCORE)
+
 def get_node_hierarchical_name(dad, tree_iter, separator="--"):
     """Get the Node Hierarchical Name"""
-    hierarchical_name = dad.treestore[tree_iter][1].replace("/", "-")
+    hierarchical_name = dad.treestore[tree_iter][1].strip()
     father_iter = dad.treestore.iter_parent(tree_iter)
     while father_iter:
-        hierarchical_name = dad.treestore[father_iter][1].replace("/", "-") + separator + hierarchical_name
+        hierarchical_name = dad.treestore[father_iter][1].strip() + separator + hierarchical_name
         father_iter = dad.treestore.iter_parent(father_iter)
+    hierarchical_name = clean_from_chars_not_for_filename(hierarchical_name)
+    if len(hierarchical_name) > cons.MAX_FILE_NAME_LEN:
+        hierarchical_name = hierarchical_name[-cons.MAX_FILE_NAME_LEN:]
     return hierarchical_name
 
 def windows_cmd_prepare_path(filepath):
@@ -80,7 +92,7 @@ def dialog_file_save_as(filename=None, filter_pattern=None, filter_name=None, cu
     if chooser.run() == gtk.RESPONSE_OK:
         filepath = chooser.get_filename()
         chooser.destroy()
-        return filepath
+        return unicode(filepath, cons.STR_UTF8, cons.STR_IGNORE) if filepath != None else None
     else:
         chooser.destroy()
         return None
@@ -108,7 +120,7 @@ def dialog_file_select(filter_pattern=None, filter_name=None, curr_folder=None,
     if chooser.run() == gtk.RESPONSE_OK:
         filepath = chooser.get_filename()
         chooser.destroy()
-        return filepath
+        return unicode(filepath, cons.STR_UTF8, cons.STR_IGNORE) if filepath != None else None
     else:
         chooser.destroy()
         return None
@@ -131,7 +143,7 @@ def dialog_folder_select(curr_folder=None, parent=None):
     if chooser.run() == gtk.RESPONSE_OK:
         folderpath = chooser.get_filename()
         chooser.destroy()
-        return folderpath
+        return unicode(folderpath, cons.STR_UTF8, cons.STR_IGNORE) if folderpath != None else None
     else:
         chooser.destroy()
         return None
@@ -168,6 +180,10 @@ def dialog_info(message, parent=None):
     dialog.run()
     dialog.destroy()
 
+def dialog_info_after_restart(parent=None):
+    """Change Only After Restart"""
+    dialog_info(_("This Change will have Effect Only After Restarting CherryTree"), parent)
+
 def dialog_warning(message, parent=None):
     """The Warning dialog"""
     dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
@@ -258,43 +274,82 @@ def set_bookmarks_menu_items(inst):
         bookmarks_menu.append(menu_item)
         inst.bookmarks_menu_items.append(menu_item)
 
+def set_menu_items_special_chars(inst):
+    """Set Special Chars menu items"""
+    if not "special_menu_1" in dir(inst):
+        inst.special_menu_1 = gtk.Menu()
+        first_run = True
+    else:
+        children_1 = inst.special_menu_1.get_children()
+        for children in children_1:
+            children.hide()
+            del children
+        first_run = False
+    for special_char in inst.special_chars:
+        menu_item = gtk.MenuItem(special_char)
+        menu_item.connect("activate", insert_special_char, special_char, inst)
+        menu_item.show()
+        inst.special_menu_1.append(menu_item)
+    if first_run:
+        # main menu
+        special_menuitem = gtk.ImageMenuItem(_("Insert _Special Character"))
+        special_menuitem.set_image(gtk.image_new_from_stock("insert", gtk.ICON_SIZE_MENU))
+        special_menuitem.set_tooltip_text(_("Insert a Special Character"))
+        special_menuitem.set_submenu(inst.special_menu_1)
+        inst.ui.get_widget("/MenuBar/EditMenu").get_submenu().insert(special_menuitem, 14)
+
 def set_menu_items_recent_documents(inst):
     """Set Recent Documents menu items on Menu and Toolbar"""
-    inst.recent_menu_1 = gtk.Menu()
-    inst.recent_menu_2 = gtk.Menu()
+    if not "recent_menu_1" in dir(inst):
+        inst.recent_menu_1 = gtk.Menu()
+        inst.recent_menu_2 = gtk.Menu()
+        first_run = True
+    else:
+        children_1 = inst.recent_menu_1.get_children()
+        children_2 = inst.recent_menu_2.get_children()
+        for children in children_1:
+            children.hide()
+            del children
+        for children in children_2:
+            children.hide()
+            del children
+        first_run = False
     for target in [1, 2]:
-        for filepath in inst.recent_docs:
+        for i, filepath in enumerate(inst.recent_docs):
+            if i >= cons.MAX_RECENT_DOCS: break
             menu_item = gtk.ImageMenuItem(filepath)
             menu_item.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
             menu_item.connect("activate", open_recent_document, filepath, inst)
             menu_item.show()
             if target == 1: inst.recent_menu_1.append(menu_item)
             else: inst.recent_menu_2.append(menu_item)
-    # main menu
-    recent_menuitem = gtk.ImageMenuItem(_("_Recent Documents"))
-    recent_menuitem.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
-    recent_menuitem.set_tooltip_text(_("Open a Recent CherryTree Document"))
-    recent_menuitem.set_submenu(inst.recent_menu_1)
-    inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 9)
-    # toolbar
-    menu_toolbutton = gtk.MenuToolButton("gtk-open")
-    menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
-    menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
-    menu_toolbutton.set_menu(inst.recent_menu_2)
-    menu_toolbutton.connect("clicked", inst.file_open)
-    inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, 9)
+    if first_run:
+        # main menu
+        recent_menuitem = gtk.ImageMenuItem(_("_Recent Documents"))
+        recent_menuitem.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
+        recent_menuitem.set_tooltip_text(_("Open a Recent CherryTree Document"))
+        recent_menuitem.set_submenu(inst.recent_menu_1)
+        inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 9)
+        # toolbar
+        menu_toolbutton = gtk.MenuToolButton("gtk-open")
+        menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
+        menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
+        menu_toolbutton.set_menu(inst.recent_menu_2)
+        menu_toolbutton.connect("clicked", inst.file_open)
+        inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, 9)
 
 def add_recent_document(inst, filepath):
     """Add a Recent Document if Needed"""
-    if not filepath in inst.recent_docs:
-        inst.recent_docs.append(filepath)
-        for target in [1, 2]:
-            menu_item = gtk.ImageMenuItem(filepath)
-            menu_item.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
-            menu_item.connect("activate", open_recent_document, filepath, inst)
-            menu_item.show()
-            if target == 1: inst.recent_menu_1.append(menu_item)
-            else: inst.recent_menu_2.append(menu_item)
+    if filepath in inst.recent_docs and inst.recent_docs[0] == filepath:
+        return
+    if filepath in inst.recent_docs: inst.recent_docs.remove(filepath)
+    inst.recent_docs.insert(0, filepath)
+    set_menu_items_recent_documents(inst)
+
+def insert_special_char(menu_item, special_char, dad):
+    """A Special character insert was Requested"""
+    if not dad.is_there_selected_node_or_error(): return
+    dad.curr_buffer.insert_at_cursor(special_char)
 
 def open_recent_document(menu_item, filepath, dad):
     """A Recent Document was Requested"""
