@@ -6,7 +6,7 @@
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
-#       the Free Software Foundation; either version 2 of the License, or
+#       the Free Software Foundation; either version 3 of the License, or
 #       (at your option) any later version.
 #
 #       This program is distributed in the hope that it will be useful,
@@ -104,7 +104,9 @@ class TablesHandler:
                     row = reader.next()
                 file_descriptor.close()
                 table_matrix.append(table_matrix.pop(0))
-                self.table_insert(iter_insert, {'col_min': 40, 'col_max': 1000, 'matrix': table_matrix})
+                self.table_insert(iter_insert, {'col_min': cons.TABLE_DEFAULT_COL_MIN,
+                                                'col_max': cons.TABLE_DEFAULT_COL_MAX,
+                                                'matrix': table_matrix})
 
     def table_insert(self, iter_insert, table=None, table_justification=None, text_buffer=None):
         """Insert a Table at the Given Iter"""
@@ -280,12 +282,12 @@ class TablesHandler:
         iter_bound.forward_char()
         self.dad.curr_buffer.delete(iter_insert, iter_bound)
         if self.dad.table_column_mode == 'rename':
-            new_label = self.dad.glade.table_column_rename_entry.get_text()
+            new_label = unicode(self.dad.glade.table_column_rename_entry.get_text(), cons.STR_UTF8, cons.STR_IGNORE)
             col_label.set_text("<b>" + new_label + "</b>")
             col_label.set_use_markup(True)
             headers[col_num] = new_label
         elif self.dad.table_column_mode == 'add':
-            headers.insert(col_num + 1, self.dad.glade.table_column_new_entry.get_text())
+            headers.insert(col_num + 1, unicode(self.dad.glade.table_column_new_entry.get_text(), cons.STR_UTF8, cons.STR_IGNORE))
             for row in table['matrix']: row.insert(col_num + 1, "")
         elif self.dad.table_column_mode == 'delete':
             headers.pop(col_num)
@@ -404,7 +406,7 @@ class UTF8Recoder:
         return self
 
     def next(self):
-        return self.reader.next().encode("utf-8")
+        return self.reader.next().encode(cons.STR_UTF8)
 
 
 class UnicodeReader:
@@ -412,14 +414,14 @@ class UnicodeReader:
     A CSV reader which will iterate over lines in the CSV file "f",
     which is encoded in the given encoding.
     """
-    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
+    def __init__(self, f, dialect=csv.excel, encoding=cons.STR_UTF8, **kwds):
         f = UTF8Recoder(f, encoding)
         self.reader = csv.reader(f, dialect=dialect, **kwds)
 
     def next(self):
         try:
             row = self.reader.next()
-            return [unicode(s, "utf-8") for s in row]
+            return [unicode(s, cons.STR_UTF8, cons.STR_IGNORE) for s in row]
         except: return None
 
     def __iter__(self):
@@ -432,7 +434,7 @@ class UnicodeWriter:
     which is encoded in the given encoding.
     """
 
-    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
+    def __init__(self, f, dialect=csv.excel, encoding=cons.STR_UTF8, **kwds):
         # Redirect output to a queue
         self.queue = cStringIO.StringIO()
         self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
@@ -440,10 +442,10 @@ class UnicodeWriter:
         self.encoder = codecs.getincrementalencoder(encoding)()
 
     def writerow(self, row):
-        self.writer.writerow([s.encode("utf-8") for s in row])
+        self.writer.writerow([s.encode(cons.STR_UTF8) for s in row])
         # Fetch UTF-8 output from the queue ...
         data = self.queue.getvalue()
-        data = data.decode("utf-8")
+        data = data.decode(cons.STR_UTF8)
         # ... and reencode it into the target encoding
         data = self.encoder.encode(data)
         # write to the target stream
